/*******************************************************************************
* File Name: HIGH_THRESH.c  
* Version 1.20
*
*  Description:
*    This file provides the source code to the API for the 8-bit Voltage DAC 
*    (VDAC8) User Module.
*
*   Note:
*     Any unusual or non-standard behavior should be noted here. Other-
*     wise, this section should remain blank.
*
*******************************************************************************
* Copyright 2008-2009, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/




#include "cytypes.h"
#include "HIGH_THRESH.h"

uint8 HIGH_THRESH_initVar = 0;

/*******************************************************************************
* Function Name: HIGH_THRESH_Start
********************************************************************************
*
* Summary:
*  The start function initializes the voltage DAC with the default values, 
*  and sets the power to the given level.  A power level of 0, is the same as executing
*  the stop function.
*
* Parameters:  
*  Power:   Sets power level between off (0) and (3) high power
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void HIGH_THRESH_Start(void) 
{
  
   /* Hardware initiazation only needs to occure the first time */
   if ( HIGH_THRESH_initVar == 0)  
   {    
     HIGH_THRESH_initVar = 1;
      HIGH_THRESH_CR0 = (HIGH_THRESH_MODE_V )  ;    

      /* Set default data source */
      if(HIGH_THRESH_DEFAULT_DATA_SRC != 0 )    
      {
          HIGH_THRESH_CR1 = (HIGH_THRESH_DEFAULT_CNTL | HIGH_THRESH_DACBUS_ENABLE) ;   
      }
      else
      {
          HIGH_THRESH_CR1 = (HIGH_THRESH_DEFAULT_CNTL | HIGH_THRESH_DACBUS_DISABLE) ;  
      } 

       /* Set default strobe mode */
      if(HIGH_THRESH_DEFAULT_STRB != 0)
      {
          HIGH_THRESH_Strobe |= HIGH_THRESH_STRB_EN ;
      }

		/* Set default range */
      HIGH_THRESH_SetRange(HIGH_THRESH_DEFAULT_RANGE); 
	  
      /* Set default speed */
      HIGH_THRESH_SetSpeed(HIGH_THRESH_DEFAULT_SPEED); 

      /* Enable power to DAC */
      HIGH_THRESH_PWRMGR |= HIGH_THRESH_ACT_PWR_EN;

      /* Set default value */
      HIGH_THRESH_SetValue(HIGH_THRESH_DEFAULT_DATA);

   }
   else
   {
       /* Enable power to DAC */
       HIGH_THRESH_PWRMGR |= HIGH_THRESH_ACT_PWR_EN;
   }

}


/*******************************************************************************
* Function Name: HIGH_THRESH_Stop
********************************************************************************
*
* Summary:
*  Powers down DAC to lowest power state.
*
* Parameters:  
*  (void)  
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void HIGH_THRESH_Stop(void)
{
   /* Disble power to DAC */
   HIGH_THRESH_PWRMGR &= ~HIGH_THRESH_ACT_PWR_EN;
}


/*******************************************************************************
* Function Name: HIGH_THRESH_SetSpeed
********************************************************************************
*
* Summary:
*  Set DAC speed
*
* Parameters:  
*  power:   Sets speed value 
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void HIGH_THRESH_SetSpeed(uint8 speed) 
{
   /* Clear power mask then write in new value */
   HIGH_THRESH_CR0 &= ~HIGH_THRESH_HS_MASK ;    
   HIGH_THRESH_CR0 |=  ( speed & HIGH_THRESH_HS_MASK) ;    
}


/*******************************************************************************
* Function Name: HIGH_THRESH_SetRange
********************************************************************************
*
* Summary:
*  Set one of three current ranges.
*
* Parameters:  
*  Range:  Sets one of Three valid ranges.
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void HIGH_THRESH_SetRange(uint8 range)
{
   HIGH_THRESH_CR0 &= ~HIGH_THRESH_RANGE_MASK ;      /* Clear existing mode */
   HIGH_THRESH_CR0 |= ( range & HIGH_THRESH_RANGE_MASK ) ; /*  Set Range  */
   HIGH_THRESH_DacTrim();
}


/*******************************************************************************
* Function Name: HIGH_THRESH_SetValue
********************************************************************************
*
* Summary:
*  Set 8-bit DAC value
*
* Parameters:  
*  value:  Sets DAC value between 0 and 255.
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void HIGH_THRESH_SetValue(uint8 value)
{
   HIGH_THRESH_Data = value;                             /*  Set Value  */
   
   #if defined(HIGH_THRESH_FIRST_SILICON)
   HIGH_THRESH_Data = value;                             /*  TODO: Remove when Silicon fix comes */
   #endif
}


/*******************************************************************************
* Function Name: HIGH_THRESH_DacTrim
********************************************************************************
*
* Summary:
*  Set the trim value for the given range.
*
* Parameters:  
*   range:  1V or 4V range.  See constants.  
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
*******************************************************************************/
void HIGH_THRESH_DacTrim(void)
{
    uint8 mode;
	
	mode = ((HIGH_THRESH_CR0 & HIGH_THRESH_RANGE_MASK) >> 2) + HIGH_THRESH_TRIM_M7_1V_RNG_OFFSET;
	
	HIGH_THRESH_TR = CY_GET_XTND_REG8((uint8 *)(HIGH_THRESH_DAC_TRIM_BASE + mode));
}


/* [] END OF FILE */

