/*******************************************************************************
* File Name: LCD.c
* Version 1.40
*
* Description:
*  This file provides source code for the Character LCD component's API.
*
* Note:
*
********************************************************************************
* Copyright (2008), Cypress Semiconductor Corporation.
********************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and
* foreign), United States copyright laws and international treaty provisions.
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the
* Cypress Source Code and derivative works for the sole purpose of creating
* custom software in support of licensee product to be used only in conjunction
* with a Cypress integrated circuit as specified in the applicable agreement.
* Any reproduction, modification, translation, compilation, or representation of
* this software except as specified above is prohibited without the express
* written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the
* materials described herein. Cypress does not assume any liability arising out
* of the application or use of any product or circuit described herein. Cypress
* does not authorize its products for use as critical components in life-support
* systems where a malfunction or failure may reasonably be expected to result in
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of
* such use and in doing so indemnifies Cypress against all charges. Use may be
* limited by and subject to the applicable Cypress software license agreement.
*******************************************************************************/

#include "LCD.h"


/*******************************************************************************
* Function Name: LCD_Start
********************************************************************************
* Summary:
*  This method does the prep work necessary to start the LCD Hardware Module.
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void LCD_Start()
{
    /* INIT CODE */
    CyDelay(40);                                                        /* Delay 15 ms */
    LCD_WrCntrlNib(LCD_DISPLAY_8_BIT_INIT);   /* Selects 8-bit mode */
    CyDelay(5);                                                         /* Delay 5 ms */
    LCD_WrCntrlNib(LCD_DISPLAY_8_BIT_INIT);   /* Selects 8-bit mode */
    CyDelay(15);                                                         /* Delay .15 ms */
    LCD_WrCntrlNib(LCD_DISPLAY_8_BIT_INIT);   /* Selects 8-bit mode */
    CyDelay(1);                                                         /* Delay 1 ms */
    LCD_WrCntrlNib(LCD_DISPLAY_4_BIT_INIT);   /* Selects 4-bit mode */
    CyDelay(5);                                                         /* Delay 4.5 ms */
    /************************************************************\
    **      Can now write full bytes to LCD in 4-bit mode       **
    \************************************************************/

    LCD_WriteControl(LCD_CURSOR_AUTO_INCR_ON);    /* Incr Cursor After Writes */
    LCD_WriteControl(LCD_DISPLAY_CURSOR_ON);      /* Turn Display, Cursor ON */
    LCD_WriteControl(LCD_DISPLAY_2_LINES_5x10);   /* 2 Lines by 5x10 Characters */
    LCD_WriteControl(LCD_DISPLAY_CURSOR_OFF);     /* Turn Display, Cursor OFF */
    LCD_WriteControl(LCD_CLEAR_DISPLAY);          /* Clear LCD Screen */
    LCD_WriteControl(LCD_DISPLAY_ON_CURSOR_OFF);  /* Turn Display ON, Cursor OFF */
    LCD_WriteControl(LCD_RESET_CURSOR_POSITION);  /* Set Cursor to 0,0 */
    CyDelay(5);                                                             /* Delay 4.5 ms */
}


/*******************************************************************************
* Function Name: LCD_Stop
********************************************************************************
* Summary:
*  Turns off the display of the LCD screen.
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void LCD_Stop()
{
    /* Calls LCD Off Macro */
    LCD_DisplayOff();
}


/*******************************************************************************
*  Function Name: LCD_Position
********************************************************************************
* Summary:
*  Moves active cursor location to a point specified by the input arguments
*
* Parameters:
*  row:     Specific row of LCD module to be written
*  column:  Column of LCD module to be written
*
* Return:
*  void
*
* Note:
*  This only applies for LCD displays which use the 2X40 address mode.
*  This results in Row 2 offset from row one by 0x28.
*  When there are more than 2 rows, each row must be fewer than 20 characters.
*
*******************************************************************************/
void LCD_Position(uint8 row, uint8 column)
{
    switch (row)
    {
        case 0:
            LCD_WriteControl(LCD_ROW_0_START + column);
            break;
        case 1:
            LCD_WriteControl(LCD_ROW_1_START + column);
            break;
        case 2:
            LCD_WriteControl(LCD_ROW_2_START + column);
            break;
        case 3:
            LCD_WriteControl(LCD_ROW_3_START + column);
            break;
        default:
            /* if default case is hit, invalid row argument was passed.*/
            break;
    }
}


/*******************************************************************************
* Function Name: LCD_PrintString
********************************************************************************
* Summary:
*  Writes a zero terminated string to the LCD.
*
* Parameters:
*  string:  pointer to head of char8 array to be written to the LCD module
*
* Return:
*  void
*
*******************************************************************************/
void LCD_PrintString(char8 * string)
{
    uint8 index_u8 = 1;
    char8 current = *string;
    /* Until null is reached, print next character */
    while(current != (char8)'\0')
    {
        LCD_WriteData(current);
        current = *(string+index_u8);
        index_u8++;
    }
}


/*******************************************************************************
*  Function Name: LCD_PutChar
********************************************************************************
* Summary:
*  Writes a single character to the current cursor position of the LCD module.
*  Custom character names (_CUSTOM_0 through
*  _CUSTOM_7) are acceptable as inputs.
*
* Parameters:
*  character:   character to be written to the LCD
*
* Return:
*  void
*
*******************************************************************************/
void LCD_PutChar(char8 character)
{
    LCD_WriteData(character);
}


/*******************************************************************************
*  Function Name: LCD_WriteData
********************************************************************************
* Summary:
*  Writes a data byte to the LCD module's Data Display RAM.
*
* Parameters:
*  dByte:   byte to be written to LCD module.
*
* Return:
*  void
*
*******************************************************************************/
void LCD_WriteData(uint8 dByte)
{
    uint8 nibble;

    nibble = dByte >> LCD_NIBBLE_SHIFT;
    nibble &= LCD_NIBBLE_MASK;
    /* Write high nibble */
    LCD_WrDatNib(nibble);

    nibble = dByte & LCD_NIBBLE_MASK;
    /* Write low nibble */
    LCD_WrDatNib(nibble);
}


/*******************************************************************************
*  Function Name: LCD_WriteControl
********************************************************************************
* Summary:
*  Writes a command byte to the LCD module.
*
* Parameters:
*  cByte:   byte to be written to LCD module.
*
* Return:
*  void
*
*******************************************************************************/
void LCD_WriteControl(uint8 cByte)
{
    uint8 nibble = cByte >> LCD_NIBBLE_SHIFT;
    LCD_IsReady();
    nibble &= LCD_NIBBLE_MASK;
    /* WrCntrlNib(High Nibble) */
    LCD_WrCntrlNib(nibble);

    nibble = cByte & LCD_NIBBLE_MASK;
    
    LCD_IsReady();
    /* WrCntrlNib(Low Nibble) */
    LCD_WrCntrlNib(nibble);
}


/*******************************************************************************
* Function Name: LCD_IsReady
********************************************************************************
* Summary:
*  Polls LCD until the ready bit is set.
*
* Parameters:
*  void
*
* Return:
*  void
*
* Note:
*  Changes pins to High-Z.
*
*******************************************************************************/
void LCD_IsReady()
{
    /* Change Port to High-Z Status on data pins */
    uint8 value = 0;
    uint8 readyValue = 0;

    /* Clear the LCD port*/
    LCD_PORT_DR &= ~LCD_PORT_MASK ;

    /* Mask off data pins to clear old values out */
    value = LCD_PORT_DM0 & ~LCD_DATA_MASK;
    /* Load in high Z values for data pins, others unchanged */
    LCD_PORT_DM0 = value | (LCD_HIGH_Z_DM0 & LCD_DATA_MASK);

    /* Mask off data pins to clear old values out */
    value = LCD_PORT_DM1 & ~LCD_DATA_MASK;
    /* Load in high Z values for data pins, others unchanged */
    LCD_PORT_DM1 = value | (LCD_HIGH_Z_DM1 & LCD_DATA_MASK);

    /* Mask off data pins to clear old values out */
    value = LCD_PORT_DM2 & ~LCD_DATA_MASK;
    /* Load in high Z values for data pins, others unchanged */
    LCD_PORT_DM2 = value | (LCD_HIGH_Z_DM2 & LCD_DATA_MASK);

    /* Make sure RS is low */
    LCD_PORT_DR &= ~LCD_RS;

    /* Set R/W high to read */
    LCD_PORT_DR |= LCD_RW;

    do
    {
        /* 40 ns delay required before rising Enable and 500ns between neighbour Enables */
        CyDelayUs(0);
        
        /* Set E high */
        LCD_PORT_DR |= LCD_E;
        
        /* 360 ns delay the setup time for data pins */
        CyDelayUs(1);
        
        /* Get port state */
        readyValue = LCD_PORT_PS;
        
        /* Set enable low */
        LCD_PORT_DR &= ~LCD_E;
        
        /* Extract ready bit */
        readyValue &= LCD_READY_BIT;
        
        /* Set E high as we in 4-bit interface we need extra oparation */
        LCD_PORT_DR |= LCD_E;
        
        /* 360 ns delay the setup time for data pins */
        CyDelayUs(1);
                
        /* Set enable low */
        LCD_PORT_DR &= ~LCD_E;

        /* Repeat until bit 4 is not zero. */
    } while (readyValue != 0);

    /* Set R/W low to write, could be next to do while loop, delay needed then */
    LCD_PORT_DR &= ~LCD_RW;

    /* Change Port to Output (Strong) on data pins */
    /* Mask off data pins to clear high z values out */
    value = LCD_PORT_DM0 & (~LCD_DATA_MASK);
    /* Load in high Z values for data pins, others unchanged */
    LCD_PORT_DM0 = value | (LCD_STRONG_DM0 & LCD_DATA_MASK);

    /* Mask off data pins to clear high z values out */
    value = LCD_PORT_DM1 & (~LCD_DATA_MASK);
    /* Load in high Z values for data pins, others unchanged */
    LCD_PORT_DM1 = value | (LCD_STRONG_DM1 & LCD_DATA_MASK);

    /* Mask off data pins to clear high z values out */
    value = LCD_PORT_DM2 & (~LCD_DATA_MASK);
    /* Load in high Z values for data pins, others unchanged */
    LCD_PORT_DM2 = value | (LCD_STRONG_DM2 & LCD_DATA_MASK);
}


/* Non API Functions */

/*******************************************************************************
*  Function Name: LCD_WrDatNib
********************************************************************************
* Summary:
*  Writes a data nibble to the LCD module.
*
* Parameters:
*  nibble:  byte containing nibble in least significant nibble to be written
*           to LCD module.
*
* Return:
*  void
*
*******************************************************************************/
void LCD_WrDatNib(uint8 nibble)
{
	uint8 dataNibble;

    LCD_IsReady();
	
	 /* RS shoul be low to select data register */
    LCD_PORT_DR |= LCD_RS;
    /* Reset RW for write operation */
    LCD_PORT_DR &= ~LCD_RW;
    
	/* Two following lines of code will give provide ua with 40ns delay */
    LCD_PORT_DR &= ~LCD_DATA_MASK;
	
    dataNibble = nibble << LCD_PORT_SHIFT;
    
    /* Write in data, bring E high*/
    LCD_PORT_DR |= (LCD_E | dataNibble);
    
    /* Minimum of 230 ns delay */
    CyDelayUs(1);
    
    LCD_PORT_DR &= ~LCD_E;
}


/*******************************************************************************
*  Function Name: LCD_WrCntrlNib
********************************************************************************
* Summary:
*  Writes a control nibble to the LCD module.
*
* Parameters:
*  nibble:  byte containing nibble in least significant nibble to be written
*           to LCD module.
*
* Return:
*  void
*
*******************************************************************************/
void LCD_WrCntrlNib(uint8 nibble)
{
	uint8 dataNibble;
    
    /* RS and RW shoul be low to select instruction register and  write operation respectively */
    LCD_PORT_DR &= ~(LCD_RS | LCD_RW);
	
	/* Two following lines of code will give provide ua with 40ns delay */
	dataNibble = (nibble & LCD_NIBBLE_MASK) << LCD_PORT_SHIFT;
    
	/* Clear data pins */
    LCD_PORT_DR &= ~LCD_DATA_MASK;
    
	/* Write control data and set enable signal */
    LCD_PORT_DR |= (LCD_E | dataNibble);
    
    /* Minimum of 230 ns delay */
    CyDelayUs(1);
    
    LCD_PORT_DR &= ~LCD_E;
}



/*******************************************************************************
*  Function Name: LCD_PrintHexUint8
********************************************************************************
* Summary:
*  Print a byte as two ASCII characters
*
* Parameters:  
*  value:   The byte to be printed out as ASCII characters.
*
* Return:
*  void
*
*******************************************************************************/
void LCD_PrintHexUint8(uint8 value)
{
    static char8 const hex[16] = "0123456789ABCDEF";
    
    LCD_PutChar((char8)hex[value>>4]);
    LCD_PutChar((char8)hex[value&0x0F]);
}


/*******************************************************************************
*  Function Name: LCD_PrintHexUint16
********************************************************************************
* Summary:
*  Print a uint16 as four ASCII characters.
*
* Parameters:  
*  value:   The uint16 to be printed out as ASCII characters.
*
* Return:
*  void
*
*******************************************************************************/
void LCD_PrintHexUint16(uint16 value)
{
    LCD_PrintHexUint8(value >> 8);
    LCD_PrintHexUint8(value & 0xFF);
}


/*******************************************************************************
*  Function Name: LCD_PrintDecUint16
********************************************************************************
* Summary:
*  Print an uint32 value as a left-justified decimal value.
*
* Parameters:  
*  value:   The byte to be printed out as ASCII characters.
*
* Return:
*  void
*
*******************************************************************************/
void LCD_PrintDecUint16(uint16 value)
{
    #define LCD_NUMBER_OF_REMAINDERS 5
    #define LCD_TEN 10

    char8 number[LCD_NUMBER_OF_REMAINDERS];
    char8 temp[LCD_NUMBER_OF_REMAINDERS]; 

    uint8 index = 0;
    uint8 numDigits = 0;

    
    /* Load these in reverse order */ 
    while(value >= LCD_TEN)
    {
        temp[index]= (value % LCD_TEN) + '0';
        value /= LCD_TEN;
        index++;
    }
    
    temp[index] = (value % LCD_TEN) + '0';
    numDigits = index;

    /* While index is greater than or equal to zero */
    while (index != 0xFFu)
    {
        number[numDigits - index] = temp[index];
        index--;
    }
    
    /* Null Termination */
    number[numDigits + 1] = (char8) 0;

    /* Print out number */
    LCD_PrintString(number);
}



/* [] END OF FILE */
