/*******************************************************************************
* File Name: SPIM.c
* Version 1.20
*
* Description:
* This file provides all API functionality of the SPI Master component
*
* Note:
*
********************************************************************************
* Copyright 2008-2009, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/



#include "cytypes.h"
#include "CyLib.h"
#include "SPIM.h"

#if (SPIM_TXBUFFERSIZE > 4)
uint16 SPIM_TXBUFFER[SPIM_TXBUFFERSIZE];
uint8 SPIM_TXBUFFERREAD = 0;
uint8 SPIM_TXBUFFERWRITE = 0;
#endif
#if (SPIM_RXBUFFERSIZE > 4)
uint16 SPIM_RXBUFFER[SPIM_RXBUFFERSIZE];
uint8 SPIM_RXBUFFERREAD = 0;
uint8 SPIM_RXBUFFERWRITE = 0;
#endif

uint8 SPIM_initvar = 0;
/*******************************************************************************
* FUNCTION NAME: SPIM_Start
********************************************************************************
*
* Summary:
* Initialize and Enable the SPI Master component
*
* Parameters:
* void
*
* Return:
* void
*
* Theory:
* Enable the clock input to enable operation
*
*******************************************************************************/
void SPIM_Start(void)
{
    if(SPIM_initvar == 0)
    {
        SPIM_initvar = 1;
        
        /*Initialize the  to Bit counter */
        SPIM_COUNTER_PERIOD = SPIM_BITCTR_INIT;
        SPIM_COUNTER_CONTROL |= SPIM_CNTR_ENABLE;
        
/********************** ISR initialization  ********************************/  
        #if(SPIM_InternalInterruptEnabled)
        
            CyIntDisable(SPIM_ISR_NUMBER);

            /* Set the ISR to point to the RTC_SUT_isr Interrupt. */
            CyIntSetVector(SPIM_ISR_NUMBER, SPIM_ISR);

            /* Set the priority. */
            CyIntSetPriority(SPIM_ISR_NUMBER, SPIM_ISR_PRIORITY);

            /* Enable it. */
            CyIntEnable(SPIM_ISR_NUMBER);   
        #endif
        
        /* Configure the Initial interrupt mask */
        /* If not using buffers then just enable the init interrupts as defined by the user */
        #if ( (SPIM_TXBUFFERSIZE <= 4) && (SPIM_TXBUFFERSIZE <= 4) )
            SPIM_STATUS_MASK  = SPIM_INIT_INTERRUPTS_MASK;
        #endif
        #if ( (SPIM_RXBUFFERSIZE > 4) )
            /* If using RX Buffer then need interrupt on RX FIFO not emtpy */
            SPIM_STATUS_MASK  = SPIM_STS_RX_FIFO_NOT_EMPTY;
        #endif
        SPIM_STATUS_ACTL |= SPIM_INT_ENABLE;
        
        /* Clear any stray data from the RX FIFO */
        SPIM_ClearFIFO();        
        
        SPIM_ReadStatus(); /* Clear any pending status bits */
    }
    
    #if(SPIM_InternalClockUsed)
        SPIM_IntClock_Enable();
    #else
        //TODO: Enable the External Clock from the information provided?
    #endif
}

/*******************************************************************************
* FUNCTION NAME: SPIM_Stop
********************************************************************************
*
* Summary:
* Disable the SPI Master component
*
* Parameters:
* -None-
*
* Return:
* -None-
*
* Theory:
* Disable the clock input to enable operation
*
* Side Effects:
* -None-
*******************************************************************************/
void SPIM_Stop(void)
{
    #if(SPIM_InternalClockUsed)
        SPIM_IntClock_Disable();
    #else
        //TODO: Enable the External Clock from the information provided?
    #endif
    
    #if(SPIM_InternalInterruptEnabled)
        /* Initialize the Interrupt Component */
        CyIntDisable(SPIM_ISR_NUMBER);
    #endif
}

/*******************************************************************************
* FUNCTION NAME: SPIM_EnableInt
********************************************************************************
*
* Summary:
* Enable interrupt generation
*
* Parameters:
* -None-
*
* Return:
* -None-
*
* Theory:
* Enable the interrupt output -or- the interrupt component itself
*
* Side Effects:
* -None-
*******************************************************************************/
void SPIM_EnableInt(void)
{
    #if(SPIM_InternalInterruptEnabled)
        CyIntEnable(SPIM_ISR_NUMBER);
    #endif
}

/*******************************************************************************
* FUNCTION NAME: SPIM_DisableInt
********************************************************************************
*
* Summary:
* Disable interrupt generation
*
* Parameters:
* -None-
*
* Return:
* -None-
*
* Theory:
* Disable the interrupt output -or- the interrupt component itself
*
* Side Effects:
* -None-
*******************************************************************************/
void SPIM_DisableInt(void)
{
    #if(SPIM_InternalInterruptEnabled)
        CyIntDisable(SPIM_ISR_NUMBER);
    #endif
}

/*******************************************************************************
* FUNCTION NAME: SPIM_SetInterruptMode
********************************************************************************
*
* Summary:
* Configure which status bits trigger an interrupt event
*
* Parameters:
* intSource: An or'd combination of the desired status bit masks (defined in
*                the header file)
*
* Return:
* -None-
*
* Theory:
* Enables the output of specific status bits to the interrupt controller
*
* Side Effects:
* -None-
*******************************************************************************/
void SPIM_SetInterruptMode(uint8 intSrc)
{
    SPIM_STATUS_MASK  = intSrc;
}

/*******************************************************************************
* FUNCTION NAME: SPIM_ReadStatus
********************************************************************************
*
* Summary:
* Read the status register for the component
*
* Parameters:
* -None-
*
* Return:
* uint8: Contents of the status register
*
* Theory:
* Allows the user and the API to read the status register for error detection
*         and flow control.
*
* Side Effects:
* -None-
*******************************************************************************/
uint8 SPIM_ReadStatus(void)
{
    return (SPIM_STATUS);
}

/*******************************************************************************
* FUNCTION NAME: SPIM_WriteByte
********************************************************************************
*
* Summary:
* Write a byte of data to be sent across the SPI
*
* Parameters:
* txDataByte: The data value to send across the SPI
*
* Return:
* -None-
*
* Theory:
* Allows the user to transmit any byte of data in a single transfer
*
* Side Effects:
* If this function is called again before the previous byte is finished then
*        the next byte will be appended to the transfer with no time between
*        the byte transfers.
*******************************************************************************/
void SPIM_WriteByte(uint16 txDataByte)
{
    
#if(SPIM_TXBUFFERSIZE > 4)

    /* Block if buffer is full, so we dont overwrite. */
    while(SPIM_TXBUFFERWRITE == (SPIM_TXBUFFERREAD - 1) ||
          (int8)(SPIM_TXBUFFERWRITE - SPIM_TXBUFFERREAD) == (int8)(SPIM_TXBUFFERSIZE - 1))
    {
        /* Software buffer is full. */
    }

    /* Disable Interrupt. */
    /* Protect variables that could change on interrupt. */
    SPIM_DisableInt();
    
    /* Add to the software buffer. */
    SPIM_TXBUFFERWRITE++;
    if(SPIM_TXBUFFERWRITE >= SPIM_TXBUFFERSIZE)
        SPIM_TXBUFFERWRITE = 0;

    SPIM_TXBUFFER[SPIM_TXBUFFERWRITE] = txDataByte;
    
    /* Enable Interrupt on TX_fifo_not Full */
    SPIM_STATUS_MASK  |= SPIM_STS_TX_FIFO_NOT_FULL;    

    /* Enable Interrupt. */
    SPIM_EnableInt();

#else /* SPIM_TXBUFFERSIZE > 4 */

   	/* Block while FIFO is full */
	while((SPIM_STATUS & SPIM_STS_TX_FIFO_NOT_FULL) == 0);
    
    /* Then write the byte */
    CY_SET_REG16(SPIM_TXDATA_PTR, txDataByte);

#endif /* SPIM_TXBUFFERSIZE > 4 */

}

/*******************************************************************************
* FUNCTION NAME: SPIM_ReadByte
********************************************************************************
*
* Summary:
* Read the next byte of data received across the SPI
*
* Parameters:
* -None-
*
* Return:
* uint16: The next byte of data read from the FIFO.
*
* Theory:
* Allows the user to read a byte of data received
*
* Side Effects:
* Will return invalid data if the FIFO is empty.  User should poll for FIFO 
*   empty status before calling Read function.
*******************************************************************************/
uint16 SPIM_ReadByte(void)
{
    uint16 rxData;

#if(SPIM_RXBUFFERSIZE > 4)

    /* Disable Interrupt. */
    /* Protect variables that could change on interrupt. */
    SPIM_DisableInt();

    if(SPIM_RXBUFFERREAD != SPIM_RXBUFFERWRITE)
    {
        SPIM_RXBUFFERREAD++;
    
        if(SPIM_RXBUFFERREAD >= SPIM_RXBUFFERSIZE)
        {
            SPIM_RXBUFFERREAD = 0;
        }
        
        rxData = SPIM_RXBUFFER[SPIM_RXBUFFERREAD];
    }
    else
    {
        rxData = CY_GET_REG16(SPIM_RXDATA_PTR);
    }

    /* Enable Interrupt. */
    SPIM_EnableInt();

#else /* SPIM_RXBUFFERSIZE > 4 */

    rxData = CY_GET_REG16(SPIM_RXDATA_PTR);

#endif /* SPIM_RXBUFFERSIZE > 4 */

	return rxData;
    
}

/*******************************************************************************
* FUNCTION NAME: SPIM_GetRxBufferSize
********************************************************************************
*
* Summary:
* Returns the number of bytes/words of data currently held in the RX buffer
*
* Parameters:
* -None-
*
* Return:
* uint8: Integer count of the number of bytes/words in the RX buffer
*
* Theory:
*
* Side Effects:
* -None-
*******************************************************************************/
uint8 SPIM_GetRxBufferSize(void)
{
    uint8 size;

#if(SPIM_RXBUFFERSIZE > 4)

    /* Disable Interrupt. */
    /* Protect variables that could change on interrupt. */
    SPIM_DisableInt();

    if(SPIM_RXBUFFERREAD == SPIM_RXBUFFERWRITE)
    {
        size = 0; /* No data in RX buffer */
    }
    else if(SPIM_RXBUFFERREAD < SPIM_RXBUFFERWRITE)
    {
        size = (SPIM_RXBUFFERWRITE - SPIM_RXBUFFERREAD);
    }
    else
    {
        size = (SPIM_RXBUFFERSIZE - SPIM_RXBUFFERREAD) + SPIM_RXBUFFERWRITE;
    }

    /* Enable interrupt. */
    SPIM_EnableInt();

#else /* SPIM_RXBUFFERSIZE < 4 */

    /* We can only know if there is data in the fifo. */
    size = (SPIM_STATUS & SPIM_STS_RX_FIFO_NOT_EMPTY) ? 1 : 0;

#endif /* SPIM_RXBUFFERSIZE < 4 */

    return size;
}

/*******************************************************************************
* FUNCTION NAME: SPIM_GetTxBufferSize
********************************************************************************
*
* Summary:
* Returns the number of bytes/words of data currently held in the TX buffer
*
* Parameters:
* -None-
*
* Return:
* uint8: Integer count of the number of bytes/words in the TX buffer
*
* Theory:
*
* Side Effects:
* -None-
*******************************************************************************/
uint8  SPIM_GetTxBufferSize(void)
{
    uint8 size;


#if(SPIM_TXBUFFERSIZE > 4)

    /* Disable Interrupt. */
    /* Protect variables that could change on interrupt. */
    SPIM_DisableInt();

    if(SPIM_TXBUFFERREAD == SPIM_TXBUFFERWRITE)
    {
        size = 0;
    }
    else if(SPIM_TXBUFFERREAD < SPIM_TXBUFFERWRITE)
    {
        size = (SPIM_TXBUFFERWRITE - SPIM_TXBUFFERREAD);
    }
    else
    {
        size = (SPIM_TXBUFFERSIZE - SPIM_TXBUFFERREAD) + SPIM_TXBUFFERWRITE;
    }

    /* Enable Interrupt. */
    SPIM_EnableInt();

#else /* SPIM_TXBUFFERSIZE > 4 */

    size = SPIM_STATUS;

    /* Is the fifo is full. */
    if(size & SPIM_STS_TX_FIFO_EMPTY)
    {
        size = 4;
    }
    else if(size & SPIM_STS_TX_FIFO_NOT_FULL)
    {
        size = 0;
    }
    else
    {
        /* We only know there is data in the fifo. */
        size = 1;
    }

#endif /* SPIM_TXBUFFERSIZE > 4 */

    return size;
}

/*******************************************************************************
* FUNCTION NAME: SPIM_ClearRxBuffer
********************************************************************************
*
* Summary:
* Clear the RX RAM buffer by setting the read and write pointers both to zero.
*
* Parameters:
* -None-
*
* Return:
* -None-
*
* Theory:
* Setting the pointers to zero makes the system believe there is no data to read
*        and writing will resume at address 0 overwriting any data that may have
*        remained in the RAM.
*
* Side Effects:
* Any received data not read from the RAM buffer will be lost when overwritten.
*******************************************************************************/
void SPIM_ClearRxBuffer(void)
{
    SPIM_ClearFIFO();
#if(SPIM_RXBUFFERSIZE > 4)
    /* Disable interrupt. */
    /* Protect variables that could change on interrupt. */
    
    SPIM_DisableInt();

    SPIM_RXBUFFERREAD = 0;
    SPIM_RXBUFFERWRITE = 0;

    /* Enable Rx interrupt. */
    SPIM_EnableInt();
#endif /* SPIM_RXBUFFERSIZE > 4 */
}

/*******************************************************************************
* FUNCTION NAME: SPIM_ClearTxBuffer
********************************************************************************
*
* Summary:
* Clear the TX RAM buffer by setting the read and write pointers both to zero.
*
* Parameters:
* -None-
*
* Return:
* -None-
*
* Theory:
* Setting the pointers to zero makes the system believe there is no data to read
*        and writing will resume at address 0 overwriting any data that may have
*        remained in the RAM.
*
* Side Effects:
* Any data not yet transmitted from the RAM buffer will be lost when overwritten
*******************************************************************************/
void SPIM_ClearTxBuffer(void)
{
#if(SPIM_TXBUFFERSIZE > 4)

    /* Disable Interrupt. */
    /* Protect variables that could change on interrupt. */
    SPIM_DisableInt();

    SPIM_TXBUFFERREAD = 0;
    SPIM_TXBUFFERWRITE = 0;

    /* Enable Interrupt. */
    SPIM_EnableInt();

#endif /* SPIM_TXBUFFERSIZE > 4 */
}

/*******************************************************************************
* FUNCTION NAME: SPIM_TxEnable
********************************************************************************
*
* Summary:
* If the SPI master is configured to use a single bi-directional pin then this
*        will set the bi-directional pin to transmit.
*
* Parameters:
* -None-
*
* Return:
* -None-
*
* Theory:
* TODO
*
* Side Effects:
* -None-
*******************************************************************************/
void SPIM_TxEnable(void)
{
    //TODO: Bi-Directional Mode is not currently supported
    //        and it will require a control register that needs to be added
    // Only used in Bi-Directional mode
    //SPIM_Control |= SPIM_CTRL_TX_SIGNAL_EN;
}

/*******************************************************************************
* FUNCTION NAME: SPIM_TxDisable
********************************************************************************
*
* Summary:
* If the SPI master is configured to use a single bi-directional pin then this
*        will set the bi-directional pin to receive.
*
* Parameters:
* -None-
*
* Return:
* -None-
*
* Theory:
* TODO
*
* Side Effects:
* -None-
*******************************************************************************/
void SPIM_TxDisable(void)
{
    //TODO: Bi-Directional Mode is not currently supported
    //        and it will require a control register that needs to be added
    // Only used in Bi-Directional mode
    //SPIM_Control &= ~SPIM_CTRL_TX_SIGNAL_EN;
}

//8-bit version
/*******************************************************************************
* FUNCTION NAME: SPIM_PutArray
********************************************************************************                       
*
* Summary:
* Write available data from ROM/RAM to the TX buffer while space is available in 
*    the TX buffer.  Keep trying until all data is passed to the TX buffer.
*
* Parameters:
* *buffer: Pointer to the location in RAM containing the data to send
* byteCount: The number of bytes to move to the transmit buffer
*
* Return:
* -None-
*
* Theory:
* TODO
*
* Side Effects:
* -None-
*******************************************************************************/
void SPIM_PutArray(uint16 *buffer, uint8 byteCount)
{
    while(byteCount > 0)
    {
        SPIM_WriteByte(*buffer++);
        byteCount--;
    }
}

//32-bit version
//void SPIM_PutArray(uint8 SSaddress, uint32 *buffer, uint16 byteCount)
//{
//}

/*******************************************************************************
* FUNCTION NAME: SPIM_ClearFIFO
********************************************************************************
*
* Summary:
* Clear the Read FIFO's of all data for a fresh start
*
* Parameters:
* None:
*
* Return:
* -None-
*
* Theory:
*
*******************************************************************************/
void SPIM_ClearFIFO (void)
{
    while( (SPIM_ReadStatus() & SPIM_STS_RX_FIFO_NOT_EMPTY) )
        SPIM_ReadByte();
}
